import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.awt.image.*;
import java.awt.geom.*;

/** The zoomTool is used to control and adjust the canvas's zooming size. 
 * It records the current canvas's zooming size and adjusts the size by listening MouseEvent.
 * The state of zoomTool depends on the current canvas's zooming size.
 *
 * It should work with all operating systems and hardware.
 * There are no variances and no security constraints.
 *
 * @author TerpPaint
 * @version 2.0
 */
public class zoomTool implements ourTool {   
    /** The current zooming size of the canvas.  Default value is 1.
     */    
    double theZoom;
    
    /** Creates a zoomTool object and sets the zoom to 1.
     * It takes in no parameters or null arguments.  It does not return anything.
     * There are no algorithms of any kind and no variances and OS dependencies.
     * There should not be any exceptions or security constraints. 
     */    
    public zoomTool() {
        super();
        theZoom = 1;
    }
    
    /** Adjusts the canvas's zooming size by listening the MouseEvent.  
     * If the left mouse button is clicked and the current canvas's zooming size
     * is less than 8.0, the new zooming size will be the twice size of the current
     * size.  If the right mouse button is clicked and the current canvas' zooming 
     * size is greater than 1.0, the new zooming size will be the half size of the current size.  
     * In addition, the new zooming size will be recorded as the state of zoomTool.
     *
     * @param mevt the mouse event which will determine the zooming size.
     * @param theCanvas the reference of main_canvas whose zooming size will be adjusted.
     */    
    public void clickAction(MouseEvent mevt, main_canvas theCanvas) {
        double currentZoom = theCanvas.getZoom();
        if (SwingUtilities.isLeftMouseButton(mevt)) {           
           if( currentZoom < 8.0 ){
               theCanvas.setZoom(currentZoom*2.0);               
               theCanvas.setOldZoom(currentZoom);               
           }
        } else {
            if( currentZoom > 1.0 ){
            	theCanvas.setZoom(currentZoom*0.5);
            	theCanvas.setOldZoom(currentZoom);
            }            
        }
        theZoom = theCanvas.getZoom();
        theCanvas.repaint();
    }
    
    /** Allows the user to drag the mouse, but is unnecessary for zooming. It was 
     * defined because zoomTool implements OurTool. 
     * @param mevt mouse event.
     * @param theCanvas the reference of main_canvas.
     */    
    public void dragAction(MouseEvent mevt, main_canvas theCanvas) {
        
    }
    
    /** Allows the user to release the mouse, but is unnecessary for zooming. It was 
     * defined because zoomTool implements OurTool. 
     * @param mevt mouse event
     * @param theCanvas the reference of main_canvas
     */    
    public void mouseReleaseAction(MouseEvent mevt, main_canvas theCanvas) {
        
    }
    
    /** Returns the value of theZoom, the state recorded by zoomTool, as an int.
     * There are no OS dependencies and variances.  No security constraints.
     * @return theZoom
     */    
    public int getZoom() { return (int)theZoom; }
       
}
